package net.jaekl.cfb.db.driver;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import net.jaekl.cfb.db.Column;
import net.jaekl.cfb.db.Condition;
import net.jaekl.cfb.db.Row;
import net.jaekl.cfb.db.Sequence;
import net.jaekl.cfb.db.SequenceMock;
import net.jaekl.cfb.db.Sort;
import net.jaekl.cfb.db.Table;
import net.jaekl.cfb.db.TableMock;

public class DbDriverMock extends DbDriver {

	private HashMap<String, TableMock> m_tables;
	private HashMap<String, SequenceMock> m_sequences;
	
	public DbDriverMock() {
		super();
		m_tables = new HashMap<String, TableMock>();
		m_sequences = new HashMap<String, SequenceMock>();
	}

	@Override
	public void load() throws ClassNotFoundException {
		// no-op
	}

	@Override
	public Connection connect(String host, int port, String dbName, String user, String pass) 
		throws SQLException 
	{
		return null;
	}
	
	@Override
	public boolean createTable(Connection con, Table table) throws SQLException
	{
		assertNotNull(con);
		assertNotNull(table);
		assertFalse(m_tables.containsKey(table.getName()));
		
		TableMock tm = new TableMock(table);
		m_tables.put(table.getName(), tm);
		return true;
	}
	
	@Override
	public void dropTable(Connection con, Table table) throws SQLException 
	{
		assertNotNull(con);
		assertNotNull(table);
		assertTrue(table instanceof TableMock);
		
		if (m_tables.containsKey(table.getName())) {
			m_tables.remove(table.getName());
		}
		else {
			throw new SQLException("Table " + table.getName() + " does not exist.");
		}
	}
	
	@Override
	public boolean createSequence(Connection con, Sequence seq) throws SQLException 
	{
		assertNotNull(con);
		assertNotNull(seq);
		assertFalse(m_sequences.containsKey(seq.getName()));
		
		SequenceMock sm = (seq instanceof SequenceMock) ? (SequenceMock)seq : new SequenceMock(seq);
		
		m_sequences.put(seq.getName(), sm);
		return true;
	}
	
	public void dropSequence(Connection con, Sequence seq) throws SQLException
	{
		assertNotNull(con);
		assertNotNull(seq);
		assertTrue(seq instanceof SequenceMock);

		if (m_sequences.containsKey(seq.getName())) {
			m_sequences.remove(seq.getName());
		}
		else {
			throw new SQLException("Sequence " + seq.getName() + " does not exist.");
		}
	}
	
	public List<Row> select(Connection con, Column[] columns, Table[] tables, Condition[] conditions, Sort[] sorts, int limit)
		throws SQLException
	{
		assertNotNull(con);
		assertNotNull(columns);
		assertNotNull(tables);
		
		// TODO:  produce sensible output
		return new ArrayList<Row>();
	}
	
	// Returns the number of rows inserted
	@Override
	public int insert(Connection con, Table table, Object[][] values) throws SQLException 
	{
		assertNotNull(con);
		assertNotNull(table);
		assertNotNull(values);
		
		TableMock tm = m_tables.get(table.getName());
		if (null == tm) {
			throw new SQLException("Table " + table.getName() + " does not exist.");
		}
		Column[] columns = tm.mock_getColumns();

		for (Object[] rowVals : values) {
			assertNotNull(rowVals);
			assertEquals(table.getNumColumns(), rowVals.length);
			
			tm.mock_insert(new Row(columns, rowVals));
		}
		
		return values.length;
	}


	public long nextVal(Connection con, Sequence seq) throws SQLException
	{
		assertNotNull(con);
		assertNotNull(seq);
		
		SequenceMock sm = m_sequences.get(seq.getName());
		return sm.mock_nextVal();
	}
	
	
	@Override
	protected String nextValSql(Sequence seq) 
	{
		return "";
	}
}
